@@ -1,7 +1,7 @@
 /*
   localize.c
 
-  (c) 1998 (W3C) MIT, INRIA, Keio University
+  (c) 1998-2000 (W3C) MIT, INRIA, Keio University
   See tidy.c for the copyright notice.
 
   You should only need to edit this file and tidy.c
@@ -11,9 +11,14 @@
 #include "platform.h"
 #include "html.h"
 
+/* used to point to Web Accessibility Guidelines */
 #define ACCESS_URL  "http://www.w3.org/WAI/GL"
 
-char *release_date = "27th September 1999";
+char *release_date = "4th August 2000";
+
+static char *currentFile; /* sasdjb 01May00 for GNU Emacs error parsing */
+
+extern uint optionerrors;
 
 /*
  This routine is the single point via which
@@ -29,7 +34,23 @@ void tidy_out(FILE *fp, const char* msg, ...)
     va_end(args);
 }
 
-void ReportTag(Lexer *lexer, Node *tag)
+void ReadingFromStdin(void)
+{
+    fprintf(stderr, "Reading markup from standard input ...\n");
+}
+
+void ShowVersion(FILE *fp)
+{
+    tidy_out(fp, "HTML Tidy release date: %s\n"
+            "See http://www.w3.org/People/Raggett for details\n", release_date);
+}
+
+void FileError(FILE *fp, const char *file)
+{
+    tidy_out(fp, "Can't open \"%s\"\n", file);
+}
+
+static void ReportTag(Lexer *lexer, Node *tag)
 {
     if (tag)
     {
@@ -46,7 +67,21 @@ void ReportTag(Lexer *lexer, Node *tag)
     }
 }
 
-void NtoS(int n, char *str)
+/* lexer is not defined when this is called */
+void ReportUnknownOption(char *option)
+{
+    optionerrors++;
+    fprintf(stderr, "Warning - unknown option: %s\n", option);
+}
+
+/* lexer is not defined when this is called */
+void ReportBadArgument(char *option)
+{
+    optionerrors++;
+    fprintf(stderr, "Warning - missing or malformed argument for option: %s\n", option);
+}
+
+static void NtoS(int n, char *str)
 {
     char buf[40];
     int i;
@@ -72,10 +107,20 @@ void NtoS(int n, char *str)
     str[n+1] = '\0';
 }
 
-void ReportPosition(Lexer *lexer)
+static void ReportPosition(Lexer *lexer)
 {
-    tidy_out(lexer->errout, "line %d", lexer->lines);
-    tidy_out(lexer->errout, " column %d - ", lexer->columns);
+    /* Change formatting to be parsable by GNU Emacs */
+    if (Emacs)
+    {
+        tidy_out(lexer->errout, "%s", currentFile);
+        tidy_out(lexer->errout, ":%d:", lexer->lines);
+        tidy_out(lexer->errout, "%d: ", lexer->columns);
+    }
+    else /* traditional format */
+    {
+        tidy_out(lexer->errout, "line %d", lexer->lines);
+        tidy_out(lexer->errout, " column %d - ", lexer->columns);
+    }
 }
 
 void ReportEncodingError(Lexer *lexer, uint code, uint c)
@@ -155,7 +200,7 @@ void ReportAttrError(Lexer *lexer, Node *node, char *attr, uint code)
         {
             tidy_out(lexer->errout, "Warning: ");
             ReportTag(lexer, node);
-            tidy_out(lexer->errout, " missing value for \"%s\" attribute", attr);
+            tidy_out(lexer->errout, " attribute \"%s\" lacks value", attr);
         }
         else if (code == MISSING_IMAGEMAP)
         {
@@ -168,7 +213,7 @@ void ReportAttrError(Lexer *lexer, Node *node, char *attr, uint code)
         {
             tidy_out(lexer->errout, "Warning: ");
             ReportTag(lexer, node);
-            tidy_out(lexer->errout, " unrecognized attribute value \"%s\"", attr);
+            tidy_out(lexer->errout, " unknown attribute value \"%s\"", attr);
         }
         else if (code == XML_ATTRIBUTE_VALUE)
         {
@@ -189,6 +234,12 @@ void ReportAttrError(Lexer *lexer, Node *node, char *attr, uint code)
             ReportTag(lexer, node);
             tidy_out(lexer->errout, " unexpected or duplicate quote mark");
         }
+        else if (code == REPEATED_ATTRIBUTE)
+        {
+            tidy_out(lexer->errout, "Warning: ");
+            ReportTag(lexer, node);
+            tidy_out(lexer->errout, " repeated attribute \"%s\"", attr);
+        }
         else if (code == PROPRIETARY_ATTR_VALUE)
         {
             tidy_out(lexer->errout, "Warning: ");
@@ -199,6 +250,12 @@ void ReportAttrError(Lexer *lexer, Node *node, char *attr, uint code)
         {
             tidy_out(lexer->errout, "Warning: end of file while parsing attributes");
         }
+        else if (code == ID_NAME_MISMATCH)
+        {
+            tidy_out(lexer->errout, "Warning: ");
+            ReportTag(lexer, node);
+            tidy_out(lexer->errout, " id and name attribute value mismatch");
+        }
 
         tidy_out(lexer->errout, "\n");
     }
@@ -243,9 +300,15 @@ void ReportWarning(Lexer *lexer, Node *element, Node *node, uint code)
             tidy_out(lexer->errout, "Warning: discarding unexpected ");
             ReportTag(lexer, node);
         }
-        else if (code == FORCED_END_ANCHOR)
+        else if (code == NESTED_EMPHASIS)
         {
-            tidy_out(lexer->errout, "Warning: <a> is probably intended as </a>");
+            tidy_out(lexer->errout, "Warning: nested emphasis ");
+            ReportTag(lexer, node);
+        }
+        else if (code == COERCE_TO_ENDTAG)
+        {
+            tidy_out(lexer->errout, "Warning: <%s> is probably intended as </%s>",
+                node->element, node->element);
         }
         else if (code == NON_MATCHING_ENDTAG)
         {
@@ -265,8 +328,20 @@ void ReportWarning(Lexer *lexer, Node *element, Node *node, uint code)
         }
         else if (code == MISSING_STARTTAG)
             tidy_out(lexer->errout, "Warning: missing <%s>", node->element);
-         else if (code == UNEXPECTED_ENDTAG)
+        else if (code == UNEXPECTED_ENDTAG)
+        {
             tidy_out(lexer->errout, "Warning: unexpected </%s>", node->element);
+
+            if (element)
+                tidy_out(lexer->errout, " in <%s>", element->element);
+        }
+        else if (code == TOO_MANY_ELEMENTS)
+        {
+            tidy_out(lexer->errout, "Warning: too many %s elements", node->element);
+
+            if (element)
+                tidy_out(lexer->errout, " in <%s>", element->element);
+        }
         else if (code == USING_BR_INPLACE_OF)
         {
             tidy_out(lexer->errout, "Warning: using <br> in place of ");
@@ -295,7 +370,11 @@ void ReportWarning(Lexer *lexer, Node *element, Node *node, uint code)
         }
         else if (code == OBSOLETE_ELEMENT)
         {
-            tidy_out(lexer->errout, "Warning: replacing obsolete element ");
+            if (element->tag && (element->tag->model & CM_OBSOLETE))
+                tidy_out(lexer->errout, "Warning: replacing obsolete element ");
+            else
+                tidy_out(lexer->errout, "Warning: replacing element ");
+
             ReportTag(lexer, element);
             tidy_out(lexer->errout, " by ");
             ReportTag(lexer, node);
@@ -331,9 +410,17 @@ void ReportWarning(Lexer *lexer, Node *element, Node *node, uint code)
         {
             tidy_out(lexer->errout, "Warning: content occurs after end of body");
         }
-        else if (code == BAD_COMMENT)
+        else if (code == MALFORMED_COMMENT)
         {
-            tidy_out(lexer->errout, "Warning: malformed comment");
+            tidy_out(lexer->errout, "Warning: adjacent hyphens within comment");
+        }
+        else if (code == BAD_COMMENT_CHARS)
+        {
+            tidy_out(lexer->errout, "Warning: expecting -- or >");
+        }
+        else if (code == BAD_XML_COMMENT)
+        {
+            tidy_out(lexer->errout, "Warning: XML comments can't contain --");
         }
         else if (code == BAD_CDATA_CONTENT)
         {
@@ -343,6 +430,10 @@ void ReportWarning(Lexer *lexer, Node *element, Node *node, uint code)
         {
             tidy_out(lexer->errout, "Warning: html namespace doesn't match content");
         }
+        else if (code == DTYPE_NOT_UPPER_CASE)
+        {
+            tidy_out(lexer->errout, "Warning: SYSTEM, PUBLIC, W3C, DTD, EN must be upper case");
+        }
         else if (code == UNEXPECTED_END_OF_FILE)
         {
             tidy_out(lexer->errout, "Warning: unexpected end of file");
@@ -379,6 +470,13 @@ void ReportError(Lexer *lexer, Node *element, Node *node, uint code)
         ReportTag(lexer, node);
         tidy_out(lexer->errout, " is not recognized!");
     }
+    else if (code == UNEXPECTED_ENDTAG)  /* generated by XML docs */
+    {
+        tidy_out(lexer->errout, "Warning: unexpected </%s>", node->element);
+
+        if (element)
+            tidy_out(lexer->errout, " in <%s>", element->element);
+    }
 
     tidy_out(lexer->errout, "\n");
 }
@@ -493,6 +591,11 @@ void ErrorSummary(Lexer *lexer)
             tidy_out(lexer->errout, "Use \"white-space: nowrap\" to inhibit wrapping in place\n");
             tidy_out(lexer->errout, "of inserting <NOBR>...</NOBR> into the markup.\n\n");
         }
+
+        if (lexer->badLayout & USING_BODY)
+        {
+            tidy_out(lexer->errout, "You are recommended to use CSS to specify page and link colors\n");
+        }
     }
 }
 
@@ -532,12 +635,43 @@ void GeneralInfo(FILE *errout)
 
 void HelloMessage(FILE *errout, char *date, char *filename)
 {
-    tidy_out(errout, "\nTidy (vers %s) Parsing \"%s\"\n", date, filename);
+    currentFile = filename;  /* for use with Gnu Emacs */
+
+    if (wstrcmp(filename, "stdin") == 0)
+        tidy_out(errout, "\nTidy (vers %s) Parsing console input (stdin)\n", date);
+    else
+        tidy_out(errout, "\nTidy (vers %s) Parsing \"%s\"\n", date, filename);
 }
 
-void ReportVersion(FILE *errout, char *filename, char *vers)
+void ReportVersion(FILE *errout, Lexer *lexer, char *filename, Node *doctype)
 {
-    tidy_out(errout, "\n\"%s\" appears to be %s\n", filename, (vers ? vers : "HTML proprietary"));
+    unsigned int i, c;
+    int state = 0;
+    char *vers = HTMLVersionName(lexer);
+
+    if (doctype)
+    {
+        tidy_out(errout, "\n%s: Doctype given is \"", filename);
+
+        for (i = doctype->start; i < doctype->end; ++i)
+        {
+            c = (unsigned char)lexer->lexbuf[i];
+
+            /* look for UTF-8 multibyte character */
+            if (c > 0x7F)
+                 i += GetUTF8((unsigned char *)lexer->lexbuf + i, &c);
+
+            if (c == '"')
+                ++state;
+            else if (state == 1)
+                putc(c, errout);
+        }
+
+        putc('"', errout);
+    }
+
+    tidy_out(errout, "\n%s: Document content looks like %s\n",
+                filename, (vers ? vers : "HTML proprietary"));
 }
 
 void ReportNumWarnings(FILE *errout, Lexer *lexer)
@@ -550,6 +684,7 @@ void ReportNumWarnings(FILE *errout, Lexer *lexer)
 
 void HelpText(FILE *out, char *prog)
 {
+#if 0  /* old style help text */
     tidy_out(out, "%s: file1 file2 ...\n", prog);
     tidy_out(out, "Utility to clean up & pretty print html files\n");
     tidy_out(out, "see http://www.w3.org/People/Raggett/tidy/\n");
@@ -570,15 +705,67 @@ void HelpText(FILE *out, char *prog)
     tidy_out(out, "  -modify or -m   to modify original files\n");
     tidy_out(out, "  -errors or -e   only show errors\n");
     tidy_out(out, "  -quiet or -q    suppress nonessential output\n");
-    tidy_out(out, "  -f <file>       write errors to <file>\n");
+    tidy_out(out, "  -f <file>       write errors to named <file>\n");
     tidy_out(out, "  -xml            use this when input is wellformed xml\n");
     tidy_out(out, "  -asxml          to convert html to wellformed xml\n");
     tidy_out(out, "  -slides         to burst into slides on h2 elements\n");
+    tidy_out(out, "  -version or -v  show version\n");
     tidy_out(out, "  -help   or -h   list command line options\n");
     tidy_out(out, "Input/Output default to stdin/stdout respectively\n");
     tidy_out(out, "Single letter options apart from -f may be combined\n");
     tidy_out(out, "as in:  tidy -f errs.txt -imu foo.html\n");
+    tidy_out(out, "You can also use --blah for any config file option blah\n");
     tidy_out(out, "For further info on HTML see http://www.w3.org/MarkUp\n");
+#endif
+    tidy_out(out, "%s: file1 file2 ...\n", prog);
+    tidy_out(out, "Utility to clean up & pretty print html files\n");
+    tidy_out(out, "see http://www.w3.org/People/Raggett/tidy/\n");
+    tidy_out(out, "options for tidy released on %s\n", release_date);
+    tidy_out(out, "\n");
+
+    tidy_out(out, "Processing directives\n");
+    tidy_out(out, "--------------------\n");
+    tidy_out(out, "  -indent or -i   indent element content\n");
+    tidy_out(out, "  -omit   or -o   omit optional endtags\n");
+    tidy_out(out, "  -wrap 72        wrap text at column 72 (default is 68)\n");
+    tidy_out(out, "  -upper  or -u   force tags to upper case (default is lower)\n");
+    tidy_out(out, "  -clean  or -c   replace font, nobr & center tags by CSS\n");
+    tidy_out(out, "  -numeric or -n  output numeric rather than named entities\n");
+    tidy_out(out, "  -errors or -e   only show errors\n");
+    tidy_out(out, "  -quiet or -q    suppress nonessential output\n");
+    tidy_out(out, "  -xml            use this when input is wellformed xml\n");
+    tidy_out(out, "  -asxml          to convert html to wellformed xml\n");
+    tidy_out(out, "  -slides         to burst into slides on h2 elements\n");
+    tidy_out(out, "\n");
+
+    tidy_out(out, "Character encodings\n");
+    tidy_out(out, "------------------\n");
+    tidy_out(out, "  -raw            leave chars > 128 unchanged upon output\n");
+    tidy_out(out, "  -ascii          use ASCII for output, Latin-1 for input\n");
+    tidy_out(out, "  -latin1         use Latin-1 for both input and output\n");
+    tidy_out(out, "  -iso2022        use ISO2022 for both input and output\n");
+    tidy_out(out, "  -utf8           use UTF-8 for both input and output\n");
+    tidy_out(out, "  -mac            use the Apple MacRoman character set\n");
+    tidy_out(out, "\n");
+    tidy_out(out, "\n");
+
+    tidy_out(out, "File manipulation\n");
+    tidy_out(out, "---------------\n");
+    tidy_out(out, "  -config <file>  set options from config file\n");
+    tidy_out(out, "  -f <file>       write errors to named <file>\n");
+    tidy_out(out, "  -modify or -m   to modify original files\n");
+    tidy_out(out, "\n");
+
+    tidy_out(out, "Miscellaneous\n");
+    tidy_out(out, "------------\n");
+    tidy_out(out, "  -version or -v  show version\n");
+    tidy_out(out, "  -help   or -h   list command line options\n");
+    tidy_out(out, "You can also use --blah for any config file option blah\n");
+    tidy_out(out, "\n");
+
+    tidy_out(out, "Input/Output default to stdin/stdout respectively\n");
+    tidy_out(out, "Single letter options apart from -f may be combined\n");
+    tidy_out(out, "as in:  tidy -f errs.txt -imu foo.html\n");
+    tidy_out(out, "For further info on HTML see http://www.w3.org/MarkUp\n");
+    tidy_out(out, "\n");
 }
-
-
