@@ -1,5 +1,5 @@
 /* mp4h -- A macro processor for HTML documents
-   Copyright 2000-2001, Denis Barbier
+   Copyright 2000-2002, Denis Barbier
    All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
@@ -158,6 +158,7 @@ typedef struct input_block input_block;
 
 /* Current input file name.  */
 char *current_file;
+char **array_current_file;
 
 /* Current input line number.  */
 int current_line;
@@ -265,7 +266,7 @@ static void
 file_clean(void)
 {
   if (debug_level & DEBUG_TRACE_INPUT)
-    DEBUG_MESSAGE2 (_("Input reverted to %s, line %d"),
+    DEBUG_MESSAGE2 ("Input reverted to %s, line %d",
                     isp->u.u_f.name, isp->u.u_f.lineno);
 
   fclose (isp->u.u_f.file);
@@ -302,7 +303,7 @@ push_file (FILE *fp, const char *title)
     }
 
   if (debug_level & DEBUG_TRACE_INPUT)
-    DEBUG_MESSAGE1 (_("Input read from %s"), title);
+    DEBUG_MESSAGE1 ("Input read from %s", title);
 
   i = (input_block *) obstack_alloc (current_input,
                                      sizeof (struct input_block));
@@ -461,8 +462,8 @@ push_string_init (void)
   if (next != NULL)
     {
       MP4HERROR ((warning_status, 0,
-        _("INTERNAL ERROR: Recursive push_string!")));
-      abort ();
+        "INTERNAL ERROR: Recursive push_string!"));
+      exit (1);
     }
 
   next = (input_block *) obstack_alloc (current_input,
@@ -597,8 +598,8 @@ init_macro_token (token_data *td)
   if (isp->funcs->read_func != macro_read)
     {
       MP4HERROR ((warning_status, 0,
-        _("INTERNAL ERROR: Bad call to init_macro_token ()")));
-      abort ();
+        "INTERNAL ERROR: Bad call to init_macro_token ()"));
+      exit (1);
     }
 
   TOKEN_DATA_TYPE (td) = TOKEN_FUNC;
@@ -637,8 +638,8 @@ next_char (void)
       else
         {
           MP4HERROR ((warning_status, 0,
-            _("INTERNAL ERROR: Input stack botch in next_char ()")));
-          abort ();
+            "INTERNAL ERROR: Input stack botch in next_char ()"));
+          exit (1);
         }
 
       /* End of input source --- pop one level.  */
@@ -674,8 +675,8 @@ peek_input (void)
       else
         {
           MP4HERROR ((warning_status, 0,
-            _("INTERNAL ERROR: Input stack botch in peek_input ()")));
-          abort ();
+            "INTERNAL ERROR: Input stack botch in peek_input ()"));
+          exit (1);
         }
 
       /* End of input source --- pop one level.  */
@@ -812,9 +813,14 @@ match_comment (const unsigned char *s)
 void
 input_init (void)
 {
+  int i;
+
   current_file = xstrdup ("NONE");
   current_line = 0;
   array_current_line = (int *) xmalloc (sizeof (int) * nesting_limit);
+  array_current_file = (char **) xmalloc (sizeof (char *) * nesting_limit);
+  for (i = 0; i < nesting_limit; i++)
+    array_current_file[i] = (char *) NULL;
 
   obstack_init (&token_stack);
   obstack_init (&input_stack);
@@ -877,7 +883,12 @@ syntax_init (void)
 void
 input_deallocate (void)
 {
+  int i;
+
   xfree ((voidstar) array_current_line);
+  for (i = 0; i < nesting_limit; i++)
+    xfree(array_current_file[i]);
+  xfree ((voidstar) array_current_file);
   xfree ((voidstar) eolcomm.string);
   xfree ((voidstar) lquote.string);
   xfree ((voidstar) rquote.string);
@@ -1140,7 +1151,7 @@ next_token (token_data *td, read_type expansion, boolean in_string)
       else if (IS_RQUOTE(ch))
         {
           MP4HERROR ((EXIT_FAILURE, 0,
-             _("INTERNAL ERROR: CHAR_RQUOTE found.")));
+             "INTERNAL ERROR: CHAR_RQUOTE found."));
         }
       else if (IS_LQUOTE(ch))             /* QUOTED STRING */
         {
@@ -1150,7 +1161,7 @@ next_token (token_data *td, read_type expansion, boolean in_string)
               ch = next_char ();
               if (ch == CHAR_EOF)
                 MP4HERROR ((EXIT_FAILURE, 0,
-                   _("INTERNAL ERROR: EOF in string")));
+                   "INTERNAL ERROR: EOF in string"));
 
               if (IS_BGROUP(ch) || IS_EGROUP(ch))
                 continue;
@@ -1193,8 +1204,8 @@ next_token (token_data *td, read_type expansion, boolean in_string)
 
             default:
               MP4HERROR ((warning_status, 0,
-                _("INTERNAL ERROR: Unknown expansion type")));
-              abort ();
+                "INTERNAL ERROR: Unknown expansion type"));
+              exit (1);
           }
         }
       else if (ch == '\\')
@@ -1251,8 +1262,8 @@ next_token (token_data *td, read_type expansion, boolean in_string)
 
             default:
               MP4HERROR ((warning_status, 0,
-                _("INTERNAL ERROR: Unknown expansion type")));
-              abort ();
+                "INTERNAL ERROR: Unknown expansion type"));
+              exit (1);
           }
         }
       else /* EVERYTHING ELSE */
@@ -1312,8 +1323,8 @@ read_file_verbatim (struct obstack *obs)
   else
     {
       MP4HERROR ((warning_status, 0,
-        _("INTERNAL ERROR: Input stack botch in read_file_verbatim ()")));
-      abort ();
+        "INTERNAL ERROR: Input stack botch in read_file_verbatim ()"));
+      exit (1);
     }
   push_string_finish (READ_BODY);
 }
@@ -1378,7 +1389,7 @@ print_token (const char *s, token_type t, token_data *td)
 
     default:
       MP4HERROR ((warning_status, 0,
-        _("INTERNAL ERROR: unknown token in print_token")));
+        "INTERNAL ERROR: unknown token in print_token"));
       break;
     }
   return 0;
