@@ -15,7 +15,7 @@
 **
 **  ======================================================================
 **
-**  Copyright (c) 1996,1997,1998 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This program is free software; it may be redistributed and/or modified
 **  only under the terms of either the Artistic License or the GNU General
@@ -37,7 +37,6 @@
 #include "eperl_global.h"
 #include "eperl_security.h"
 #include "eperl_getopt.h"
-#include "eperl_perl5.h"
 #include "eperl_proto.h"
 
 #define _EPERL_VERSION_C_AS_HEADER_
@@ -60,7 +59,8 @@ void PrintError(int mode, char *scripturl, char *scriptfile, char *logfile, char
     char *cp;
 
     va_start(ap, str);
-    vsprintf(ca, str, ap);
+    vsnprintf(ca, sizeof(ca), str, ap);
+    ca[sizeof(ca)-1] = NUL;
 
     IO_restore_stdout();
     IO_restore_stderr();
@@ -68,7 +68,7 @@ void PrintError(int mode, char *scripturl, char *scriptfile, char *logfile, char
     if (mode == MODE_CGI || mode == MODE_NPHCGI) {
         if (mode == MODE_NPHCGI)
             HTTP_PrintResponseHeaders("");
-        printf("Content-Type: text/html\n\n");
+        printf("Content-Type: text/html\r\n\r\n");
         printf("<html>\n");
         printf("<head>\n");
         printf("<title>ePerl: ERROR: %s</title>\n", ca);
@@ -139,7 +139,7 @@ void give_version(void)
 {
     fprintf(stdout, "%s\n", eperl_version.v_tex);
     fprintf(stdout, "\n");
-    fprintf(stdout, "Copyright (c) 1996-2000 Ralf S. Engelschall <rse@engelschall.com>\n");
+    fprintf(stdout, "Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>\n");
     fprintf(stdout, "\n");
     fprintf(stdout, "This program is distributed in the hope that it will be useful,\n");
     fprintf(stdout, "but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
@@ -148,20 +148,6 @@ void give_version(void)
     fprintf(stdout, "\n");
 }
 
-void give_version_extended(void)
-{
-    give_version();
-    fprintf(stdout, "Characteristics of this binary:\n");
-    fprintf(stdout, "  Perl Version    : %s (%s)\n", AC_perl_vers, AC_perl_prog);
-    fprintf(stdout, "  Perl I/O Layer  : %s\n", PERL_IO_LAYER_ID);
-    fprintf(stdout, "  Perl Library    : %s/CORE/libperl.a\n", AC_perl_archlib);
-    fprintf(stdout, "  Perl DynaLoader : %s\n", AC_perl_dla);
-    fprintf(stdout, "  System Libs     : %s\n", AC_perl_libs);
-    fprintf(stdout, "  Built User      : %s\n", AC_build_user);
-    fprintf(stdout, "  Built Time      : %s\n", AC_build_time_iso);
-    fprintf(stdout, "\n");
-}
-
 void give_readme(void)
 {
     fprintf(stdout, ePerl_README);
@@ -176,7 +162,7 @@ void give_img_logo(void)
 {
     if (mode == MODE_NPHCGI)
         HTTP_PrintResponseHeaders("");
-    printf("Content-Type: image/gif\n\n");
+    printf("Content-Type: image/gif\r\n\r\n");
     fwrite(ePerl_LOGO_data, ePerl_LOGO_size, 1, stdout);
 }
 
@@ -184,7 +170,7 @@ void give_img_powered(void)
 {
     if (mode == MODE_NPHCGI)
         HTTP_PrintResponseHeaders("");
-    printf("Content-Type: image/gif\n\n");
+    printf("Content-Type: image/gif\r\n\r\n");
     fwrite(ePerl_POWERED_data, ePerl_POWERED_size, 1, stdout);
 }
 
@@ -310,8 +296,6 @@ int main(int argc, char **argv, char **env)
 {
     DECL_EXRC;
     FILE *fp = NULL;
-    FILE *er = NULL;
-    FILE *out = NULL;
     char *cpBuf = NULL;
     char *cpBuf2 = NULL;
     char *cpBuf3 = NULL;
@@ -330,14 +314,13 @@ int main(int argc, char **argv, char **env)
     char *source = NULL;
     char sourcedir[2048];
     char *cp;
-    static PerlInterpreter *my_perl = NULL; 
     struct stat st;
+    char *cpOut0 = NULL;
     char *cpOut = NULL;
-    int size;
     struct passwd *pw;
     struct passwd *pw2;
     struct group *gr;
-    int uid = 0, gid = 0;
+    int uid, gid;
     int keepcwd = FALSE;
     int c;
     char *cpScript = NULL;
@@ -450,7 +433,7 @@ int main(int argc, char **argv, char **env)
                 give_version();
                 myexit(EX_OK);
             case 'V':
-                give_version_extended();
+                give_version_extended_perl();
                 myexit(EX_OK);
             case 'h':
                 give_usage(progname);
@@ -525,7 +508,8 @@ int main(int argc, char **argv, char **env)
 
         /*  determine whether pure CGI or NPH-CGI mode */ 
         if ((cp = getenv("SCRIPT_FILENAME")) != NULL) { 
-            strcpy(ca, cp);
+            strncpy(ca, cp, sizeof(ca));
+            ca[sizeof(ca)-1] = NUL;
             if ((cp = strrchr(ca, '/')) != NULL) 
                 *cp++ = NUL;
             else 
@@ -540,7 +524,8 @@ int main(int argc, char **argv, char **env)
         }
 
         /* set the command line for ``ps'' output */
-        sprintf(ca, "%s %s [%sCGI/SSSL]", argv[0], source, mode == MODE_NPHCGI ? "NPH-" : "");
+        snprintf(ca, sizeof(ca), "%s %s [%sCGI/SSSL]", argv[0], source, mode == MODE_NPHCGI ? "NPH-" : "");
+        ca[sizeof(ca)-1] = NUL;
         argv[0] = strdup(ca);
     }
     /*
@@ -585,7 +570,8 @@ int main(int argc, char **argv, char **env)
 
         /*  determine whether pure CGI or NPH-CGI mode */ 
         if ((cp = getenv("SCRIPT_FILENAME")) != NULL) { 
-            strcpy(ca, cp);
+            strncpy(ca, cp, sizeof(ca));
+            ca[sizeof(ca)-1] = NUL;
             if ((cp = strrchr(ca, '/')) != NULL) 
                 *cp++ = NUL;
             else 
@@ -600,7 +586,8 @@ int main(int argc, char **argv, char **env)
         }
 
         /* set the command line for ``ps'' output */
-        sprintf(ca, "%s %s [%sCGI/stand-alone]", argv[0], source, mode == MODE_NPHCGI ? "NPH-" : "");
+        snprintf(ca, sizeof(ca), "%s %s [%sCGI/stand-alone]", argv[0], source, mode == MODE_NPHCGI ? "NPH-" : "");
+        ca[sizeof(ca)-1] = NUL;
         argv[0] = strdup(ca);
     }
     /*
@@ -840,7 +827,8 @@ int main(int argc, char **argv, char **env)
                     dir_home = getcwd(NULL, 1024);
 
                     /* determine physical dir of file */
-                    strcpy(dir_tmp, source);
+                    strncpy(dir_tmp, source, sizeof(dir_tmp));
+                    dir_tmp[sizeof(dir_tmp)-1] = NUL;
                     if ((cp = strrchr(dir_tmp, '/')) == NULL) {
                         if (DO_FOR_FAILED_STEP == STOP_AND_ERROR) {
                             PrintError(mode, source, NULL, NULL, "Invalid script ``%s'': no absolute path", source);
@@ -947,8 +935,9 @@ int main(int argc, char **argv, char **env)
         cpPort = getenv("SERVER_PORT");
         if (stringEQ(cpPort, "80"))
             cpPort = NULL;
-        sprintf(ca, "http://%s%s%s%s", 
+        snprintf(ca, sizeof(ca), "http://%s%s%s%s", 
                 cpHost, cpPort != NULL ? ":" : "", cpPort != NULL ? cpPort : "", cpPath);
+        ca[sizeof(ca)-1] = NUL;
         env = mysetenv(env, "SCRIPT_SRC_URL", "%s", ca);
         env = mysetenv(env, "SCRIPT_SRC_URL_FILE", "%s", filename(ca));
         env = mysetenv(env, "SCRIPT_SRC_URL_DIR", "%s", dirname(ca));
@@ -977,7 +966,8 @@ int main(int argc, char **argv, char **env)
     if (fPP) {
         /* switch to directory where script stays */
         getcwd(cwd, MAXPATHLEN);
-        strcpy(sourcedir, source);
+        strncpy(sourcedir, source, sizeof(sourcedir));
+        sourcedir[sizeof(sourcedir)-1] = NUL;
         for (cp = sourcedir+strlen(sourcedir); cp > sourcedir && *cp != '/'; cp--)
             ;
         *cp = NUL;
@@ -1000,7 +990,8 @@ int main(int argc, char **argv, char **env)
     cpScript = cpBuf2;
 
     /* write buffer to temporary script file */
-    strcpy(perlscript, mytmpfile("ePerl.script"));
+    strncpy(perlscript, mytmpfile("ePerl.script"), sizeof(perlscript));
+    perlscript[sizeof(perlscript)-1] = NUL;
 #ifndef DEBUG_ENABLED
     unlink(perlscript);
 #endif
@@ -1027,41 +1018,19 @@ int main(int argc, char **argv, char **env)
         fclose(fp); fp = NULL;
     }
 
-    /* open a file for Perl's STDOUT channel
-       and redirect stdout to the new channel */
-    strcpy(perlstdout, mytmpfile("ePerl.stdout"));
+    /* temporary filename for Perl's STDOUT channel */
+    strncpy(perlstdout, mytmpfile("ePerl.stdout"), sizeof(perlstdout));
+    perlstdout[sizeof(perlstdout)-1] = NUL;
 #ifndef DEBUG_ENABLED
     unlink(perlstdout);
 #endif
-    if ((out = fopen(perlstdout, "w")) == NULL) {
-        PrintError(mode, source, NULL, NULL, "Cannot open STDOUT file `%s' for writing", perlstdout);
-        CU(mode == MODE_FILTER ? EX_IOERR : EX_OK);
-    }
-    IO_redirect_stdout(out);
 
-    /* open a file for Perl's STDERR channel 
-       and redirect stderr to the new channel */
-    strcpy(perlstderr, mytmpfile("ePerl.stderr"));
+    /* temporary filename for Perl's STDERR channel */
+    strncpy(perlstderr, mytmpfile("ePerl.stderr"), sizeof(perlstderr));
+    perlstderr[sizeof(perlstderr)-1] = NUL;
 #ifndef DEBUG_ENABLED
     unlink(perlstderr);
 #endif
-    if ((er = fopen(perlstderr, "w")) == NULL) {
-        PrintError(mode, source, NULL, NULL, "Cannot open STDERR file `%s' for writing", perlstderr);
-        CU(mode == MODE_FILTER ? EX_IOERR : EX_OK);
-    }
-    IO_redirect_stderr(er);
-
-    /*  initialise the Perl Locale environment  */
-#if AC_perl_vnum < 500400
-    perl_init_i18nl14n(1); /* Perl 5.003 or lower */
-#else
-    perl_init_i18nl10n(1); /* Perl 5.004 or higher */
-#endif
-
-    /*  now allocate the Perl interpreter  */
-    my_perl = perl_alloc();   
-    perl_construct(my_perl); 
-    /* perl_destruct_level = 1; */
 
     /*  create command line...  */
     myargc = 0;
@@ -1079,105 +1048,43 @@ int main(int argc, char **argv, char **env)
     /*  - and the script itself  */
     myargv[myargc++] = perlscript;   
 
-    /*  now parse the script! 
-        NOTICE: At this point, the script gets 
-        only _parsed_, not evaluated/executed!  */
-#ifdef HAVE_PERL_DYNALOADER
-    rc = perl_parse(my_perl, Perl5_XSInit, myargc, myargv, env);
-#else
-    rc = perl_parse(my_perl, NULL, myargc, myargv, env);
-#endif
-    if (rc != 0) { 
-        if (fCheck && mode == MODE_FILTER) {
-            fclose(er); er = NULL;
-            IO_restore_stdout();
-            IO_restore_stderr();
-            if ((cpBuf = ePerl_ReadErrorFile(perlstderr, perlscript, source)) != NULL) {
-                fprintf(stderr, cpBuf);
-            }
-            CU(EX_IOERR);
-        }
-        else {
-            fclose(er); er = NULL;
-            PrintError(mode, source, perlscript, perlstderr, "Perl parsing error (interpreter rc=%d)", rc);
-            CU(mode == MODE_FILTER ? EX_IOERR : EX_OK);
-        }
-    }
-
-    /* Stop when we are just doing a syntax check */
-    if (fCheck && mode == MODE_FILTER) {
-        fclose(er); er = NULL;
-        IO_restore_stdout();
-        IO_restore_stderr();
-        fprintf(stderr, "%s syntax OK\n", source);
-        CU(EX_OK);
-    }
-
-    /* change to directory of script:
-       this actually is not important to us, but really useful 
-       for the ePerl source file programmer!! */
-    cwd[0] = NUL;
-    if (!keepcwd) {
-        /* if running as a Unix filter remember the cwd for outputfile */
-        if (mode == MODE_FILTER)
-            getcwd(cwd, MAXPATHLEN);
-        /* determine dir of source file and switch to it */
-        strcpy(sourcedir, source);
-        for (cp = sourcedir+strlen(sourcedir); cp > sourcedir && *cp != '/'; cp--)
-            ;
-        *cp = NUL;
-        chdir(sourcedir);
-    }
-
-    /*  Set the previously remembered Perl 5 scalars (option -d) */
-    Perl5_SetRememberedScalars();
-
-    /*  Force unbuffered I/O */
-    Perl5_ForceUnbufferedStdout();
-
-    /*  NOW IT IS TIME to evaluate/execute the script!!! */
-    rc = perl_run(my_perl);
-
-    /*  pre-close the handles, to be able to check
-        its size and to be able to display the contents */
-    fclose(out); out = NULL;
-    fclose(er);  er  = NULL;
-
-    /*  when the Perl interpreter failed or there
-        is data on stderr, we print a error page */
-    if (stat(perlstderr, &st) == 0)
-        size = st.st_size;
-    else
-        size = 0;
-    if (rc != 0 || size > 0) {
-        PrintError(mode, source, perlscript, perlstderr, "Perl runtime error (interpreter rc=%d)", rc);
-        CU(mode == MODE_FILTER ? EX_IOERR : EX_OK);
+    rc = Perl5_Run(myargc, myargv, mode, fCheck, keepcwd, source, env, perlscript, perlstderr, perlstdout);
+    /*  Return code:
+     *     0: ok
+     *    -1: fCheck && mode == MODE_FILTER and
+     *        no error detected by perl_parse()
+     *    otherwise: error detected by perl_parse() or perl_run()
+     *  Error message has already been delivered bu Perl5_Run.
+     */
+    if (rc != 0) {
+        if (rc == -1)
+            CU(EX_OK);
+        CU(mode == MODE_FILTER ? EX_FAIL : EX_OK);
     }
 
     /*  else all processing was fine, so
         we read in the stdout contents */
     if ((cpOut = ePerl_ReadSourceFile(perlstdout, &cpOut, &nOut)) == NULL) {
         PrintError(mode, source, NULL, NULL, "Cannot open STDOUT file `%s' for reading", perlstdout);
-        CU(mode == MODE_FILTER ? EX_IOERR : EX_OK);
+        CU(mode == MODE_FILTER ? EX_FAIL : EX_OK);
     }
+    cpOut0 = cpOut; /* cpOut will move by HTTP_PrintResponseHeaders() later */
     stat(perlstdout, &st);
 
-    /* ok, now recover the stdout and stderr and
-       print out the real contents on stdout or outputfile */
-    IO_restore_stdout();
-    IO_restore_stderr();
-
     /*  if we are running as a NPH-CGI/1.1 script
         we had to provide the HTTP reponse headers ourself */
     if (mode == MODE_NPHCGI) {
-        HTTP_PrintResponseHeaders(cpOut);
+        char *p = HTTP_PrintResponseHeaders(cpOut);
+        /* HTTP_PrintResponseHeader will skip HTTP status line */
+        nOut -= (p - cpOut); /* adjust length */
+        cpOut = p; /* points top of HTTP response header */
 
         /* if there are no HTTP header lines, we print a basic
            Content-Type header which should be ok */
         if (!HTTP_HeadersExists(cpOut)) {
-            printf("Content-Type: text/html\n");
-            printf("Content-Length: %d\n", nOut);
-            printf("\n");
+            printf("Content-Type: text/html\r\n");
+            printf("Content-Length: %d\r\n", nOut);
+            printf("\r\n");
         }
     }
     else if (mode == MODE_CGI) {
@@ -1221,18 +1128,7 @@ int main(int argc, char **argv, char **env)
 
     CUS: /* the Clean Up Sequence */
 
-    /* Ok, the script got evaluated. Now we can destroy 
-       and de-allocate the Perl interpreter */
-    if (my_perl) {
-       perl_destruct(my_perl);                                                    
-       perl_free(my_perl);
-    }
-
     /* close all still open file handles */
-    if (out)
-        fclose(out);
-    if (er)
-        fclose(er);
     if (fp)
         fclose(fp);
 
@@ -1241,8 +1137,8 @@ int main(int argc, char **argv, char **env)
         free(cpBuf);
     if (cpBuf2)
         free(cpBuf2);
-    if (cpOut)
-        free(cpOut);
+    if (cpOut0)
+        free(cpOut0);
 
     /* remove temporary files */
 #ifndef DEBUG_ENABLED
