@@ -15,7 +15,7 @@
 **
 **  ======================================================================
 **
-**  Copyright (c) 1996,1997,1998 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This program is free software; it may be redistributed and/or modified
 **  only under the terms of either the Artistic License or the GNU General
@@ -66,7 +66,8 @@ void ePerl_SetError(char *str, ...)
     va_list ap;
 
     va_start(ap, str);
-    vsprintf(ePerl_ErrorString, str, ap);
+    vsnprintf(ePerl_ErrorString, sizeof(ePerl_ErrorString), str, ap);
+    ePerl_ErrorString[sizeof(ePerl_ErrorString)-1] = NUL;
     va_end(ap);
     return;
 }
@@ -79,6 +80,7 @@ char *ePerl_GetError(void)
     return ePerl_ErrorString;
 }
 
+#if 0 /* unsafe ? */
 /*
 **  fprintf for internal buffer 
 */
@@ -91,10 +93,26 @@ char *ePerl_fprintf(char *cpOut, char *str, ...)
     va_end(ap);
     return cpOut+strlen(cpOut);
 }
+#else /* safe version */
+char *ePerl_fnprintf(char *cpOut, int *n, char *str, ...)
+{
+    va_list ap;
+
+    if (*n <= 0) { abort(); } /* hope NEVER */
+    va_start(ap, str);
+    vsnprintf(cpOut, *n, str, ap);
+    cpOut[*n - 1] = NUL;
+    va_end(ap);
+    *n -= strlen(cpOut);
+    if (*n <= 0) { abort(); } /* hope NEVER */
+    return cpOut+strlen(cpOut);
+}
+#endif
 
 /*
 **  fwrite for internal buffer 
 */
+#if 0 /* unsafe */
 char *ePerl_fwrite(char *cpBuf, int nBuf, int cNum, char *cpOut)
 {
     char *cp;
@@ -106,7 +124,24 @@ char *ePerl_fwrite(char *cpBuf, int nBuf, int cNum, char *cpOut)
     *cp = NUL;
     return cp;
 }
+#else /* safe */
+char *ePerl_fnwrite(char *cpBuf, int nBuf, int cNum, char *cpOut, int *cpOutLen)
+{
+    char *cp;
+    int n;
 
+    n = nBuf*cNum;
+    if (*cpOutLen < n) { abort(); }
+    (void)strncpy(cpOut, cpBuf, n);
+    cpOut[*cpOutLen - 1] = NUL;
+    cp = cpOut + n;
+    *cp = NUL;
+    *cpOutLen -= n;
+    return cp;
+}
+#endif
+
+#if 0 /* unsafe */
 /*
 **  fwrite for internal buffer WITH character escaping
 */
@@ -129,6 +164,29 @@ char *ePerl_Efwrite(char *cpBuf, int nBuf, int cNum, char *cpOut)
     *cpO = NUL;
     return cpO;
 }
+#else /* safe */
+char *ePerl_Efnwrite(char *cpBuf, int nBuf, int cNum, char *cpOut, int *n)
+{
+    char *cpI;
+    char *cpO;
+
+    if (*n <= 0) { abort(); }
+    for (cpI = cpBuf, cpO = cpOut; cpI < (cpBuf+(nBuf*cNum)); ) {
+        switch (*cpI) {
+            case '"':  *cpO++ = '\\'; *cpO++ = *cpI++; *n -= 2;    break;
+            case '@':  *cpO++ = '\\'; *cpO++ = *cpI++; *n -= 2;    break;
+            case '$':  *cpO++ = '\\'; *cpO++ = *cpI++; *n -= 2;    break;
+            case '\\': *cpO++ = '\\'; *cpO++ = *cpI++; *n -= 2;    break;
+            case '\t': *cpO++ = '\\'; *cpO++ = 't'; cpI++; *n -= 2;break;
+            case '\n': *cpO++ = '\\'; *cpO++ = 'n'; cpI++; *n -= 2;break;
+            default: *cpO++ = *cpI++; *n -= 1;
+        }
+        if (*n <= 0) { abort(); }
+    }
+    *cpO = NUL;
+    return cpO;
+}
+#endif
 
 
 /*
@@ -247,6 +305,7 @@ static struct html2char html2char[] = {
     { NULL, NUL }
 };
 
+#if 0 /* unsafe */
 char *ePerl_Cfwrite(char *cpBuf, int nBuf, int cNum, char *cpOut)
 {
     char *cpI;
@@ -278,6 +337,42 @@ char *ePerl_Cfwrite(char *cpBuf, int nBuf, int cNum, char *cpOut)
     *cpO = NUL;
     return cpO;
 }
+#else /* safe */
+char *ePerl_Cfnwrite(char *cpBuf, int nBuf, int cNum, char *cpOut, int *cpOutLen)
+{
+    char *cpI;
+    char *cpO;
+    int i;
+    int n;
+    char *cpE;
+
+    if (*cpOutLen <= 0) { abort(); }
+    cpI = cpBuf;
+    cpO = cpOut;
+    cpE = cpBuf+(nBuf*cNum);
+    while (cpI < cpE) {
+        if (*cpI == '&') {
+            for (i = 0; html2char[i].h != NULL; i++) {
+                n = strlen(html2char[i].h);
+                if (cpI+1+n+1 < cpE) {
+                    if (*(cpI+1+n) == ';') {
+                        if (strncmp(cpI+1, html2char[i].h, n) == 0) {
+                            *cpO++ = html2char[i].c;
+                            if (--*cpOutLen <= 0) { abort(); }
+                            cpI += 1+n+1;
+                            continue;
+                        }
+                    }
+                }
+            }
+        }
+        *cpO++ = *cpI++;
+        if (--*cpOutLen <= 0) { abort(); }
+    }
+    *cpO = NUL;
+    return cpO;
+}
+#endif
 
 
 /*
@@ -326,15 +421,6 @@ char *ep_strncasestr(char *buf, char *str, int n)
     return NULL;
 }
 
-char *ep_strndup(char *buf, int n)
-{
-    char *cp;
-
-    cp = (char *)malloc(n+1);
-    strncpy(cp, buf, n);
-    return cp;
-}
-
 
 /*
 **  convert buffer from bristled format to plain format
@@ -344,6 +430,7 @@ char *ePerl_Bristled2Plain(char *cpBuf)
     char *rc;
     char *cpOutBuf = NULL;
     char *cpOut = NULL;
+    int cpOutLen = 0;
     char *cps, *cpe;
     char *cps2, *cpe2;
     int nBuf;
@@ -369,6 +456,7 @@ char *ePerl_Bristled2Plain(char *cpBuf)
         CU(NULL);
     }
     cpOut = cpOutBuf;
+    cpOutLen = n;
 
     /* now step through the file and convert it to legal Perl code.
        This is a bit complicated because we have to make sure that
@@ -393,25 +481,25 @@ char *ePerl_Bristled2Plain(char *cpBuf)
                 while (cps2 < cpEND && (cpe2 = ep_strnchr(cps2, '\n', cpEND-cps2)) != NULL) {
                     if (ePerl_line_continuation && cps < cpe2 && *(cpe2-1) == '\\') {
                         if (cpe2-1-cps2 > 0) {
-                            cpOut = ePerl_fprintf(cpOut, "print \"");
-                            cpOut = ePerl_Efwrite(cps2, cpe2-1-cps2, 1, cpOut);
-                            cpOut = ePerl_fprintf(cpOut, "\";");
+                            cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "print \"");
+                            cpOut = ePerl_Efnwrite(cps2, cpe2-1-cps2, 1, cpOut, &cpOutLen);
+                            cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\";");
                         }
-                        cpOut = ePerl_fprintf(cpOut, "\n");
+                        cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\n");
                     }
                     else {
-                        cpOut = ePerl_fprintf(cpOut, "print \"");
-                        cpOut = ePerl_Efwrite(cps2, cpe2-cps2, 1, cpOut);
-                        cpOut = ePerl_fprintf(cpOut, "\\n\";\n");
+                        cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "print \"");
+                        cpOut = ePerl_Efnwrite(cps2, cpe2-cps2, 1, cpOut, &cpOutLen);
+                        cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\\n\";\n");
                     }
                     cps2 = cpe2+1;
                 }
                 /* then do the remainder which is not
                    finished by a newline */
                 if (cpEND > cps2) {
-                    cpOut = ePerl_fprintf(cpOut, "print \"");
-                    cpOut = ePerl_Efwrite(cps2, cpEND-cps2, 1, cpOut);
-                    cpOut = ePerl_fprintf(cpOut, "\";");
+                    cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "print \"");
+                    cpOut = ePerl_Efnwrite(cps2, cpEND-cps2, 1, cpOut, &cpOutLen);
+                    cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\";");
                 }
             }
             break; /* and break the whole processing step */
@@ -428,30 +516,30 @@ char *ePerl_Bristled2Plain(char *cpBuf)
                 while ((cpe2 = ep_strnchr(cps2, '\n', cpe-cps2)) != NULL) {
                     if (ePerl_line_continuation && cps < cpe2 && *(cpe2-1) == '\\') {
                         if (cpe2-1-cps2 > 0) {
-                            cpOut = ePerl_fprintf(cpOut, "print \"");
-                            cpOut = ePerl_Efwrite(cps2, cpe2-1-cps2, 1, cpOut);
-                            cpOut = ePerl_fprintf(cpOut, "\";");
+                            cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "print \"");
+                            cpOut = ePerl_Efnwrite(cps2, cpe2-1-cps2, 1, cpOut, &cpOutLen);
+                            cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\";");
                         }
-                        cpOut = ePerl_fprintf(cpOut, "\n");
+                        cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\n");
                     }
                     else {
-                        cpOut = ePerl_fprintf(cpOut, "print \"");
-                        cpOut = ePerl_Efwrite(cps2, cpe2-cps2, 1, cpOut);
-                        cpOut = ePerl_fprintf(cpOut, "\\n\";\n");
+                        cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "print \"");
+                        cpOut = ePerl_Efnwrite(cps2, cpe2-cps2, 1, cpOut, &cpOutLen);
+                        cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\\n\";\n");
                     }
                     cps2 = cpe2+1;
                 }
                 if (cpe > cps2) {
-                    cpOut = ePerl_fprintf(cpOut, "print \"");
-                    cpOut = ePerl_Efwrite(cps2, cpe-cps2, 1, cpOut);
-                    cpOut = ePerl_fprintf(cpOut, "\";");
+                    cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "print \"");
+                    cpOut = ePerl_Efnwrite(cps2, cpe-cps2, 1, cpOut, &cpOutLen);
+                    cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\";");
                 }
             }
 
             /* just output a leading space to make
                the -x display more readable. */
             if (cpOut > cpOutBuf && *(cpOut-1) != '\n') 
-                cpOut = ePerl_fprintf(cpOut, " ");
+                cpOut = ePerl_fnprintf(cpOut, &cpOutLen, " ");
 
             /* skip the start delimiter */
             cps = cpe+strlen(ePerl_begin_delimiter);
@@ -460,7 +548,7 @@ char *ePerl_Bristled2Plain(char *cpBuf)
              * e.g. <:=$var:>
              */
             if (*cps == '=') {
-                cpOut = ePerl_fprintf(cpOut, "print ");
+                cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "print ");
                 cps++;
             }
 
@@ -493,16 +581,16 @@ char *ePerl_Bristled2Plain(char *cpBuf)
             /* pass through the ePerl block without changes! */
             if (cpe2 > cps) { 
                 if (ePerl_convert_entities == TRUE)
-                    cpOut = ePerl_Cfwrite(cps, cpe2-cps, 1, cpOut);
+                    cpOut = ePerl_Cfnwrite(cps, cpe2-cps, 1, cpOut, &cpOutLen);
                 else
-                    cpOut = ePerl_fwrite(cps, cpe2-cps, 1, cpOut);
+                    cpOut = ePerl_fnwrite(cps, cpe2-cps, 1, cpOut, &cpOutLen);
 
                 /* be smart and automatically add a semicolon
                    if not provided at the end of the ePerl block.
                    But know the continuation indicator "_". */
                 if ((*(cpe2-1) != ';') &&
                     (*(cpe2-1) != '_')   )
-                    cpOut = ePerl_fprintf(cpOut, ";");
+                    cpOut = ePerl_fnprintf(cpOut, &cpOutLen, ";");
                 if (*(cpe2-1) == '_') 
                     cpOut = cpOut - 1;
             }
@@ -510,13 +598,13 @@ char *ePerl_Bristled2Plain(char *cpBuf)
             /* end preserve newlines for correct line numbers */
             for ( ; cpe2 <= cpe; cpe2++)
                 if (*cpe2 == '\n')
-                    cpOut = ePerl_fprintf(cpOut, "\n");
+                    cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\n");
 
             /* output a trailing space to make
                the -x display more readable when
                no newlines have finished the block. */
             if (cpOut > cpOutBuf && *(cpOut-1) != '\n') 
-                cpOut = ePerl_fprintf(cpOut, " ");
+                cpOut = ePerl_fnprintf(cpOut, &cpOutLen, " ");
 
             /* and adjust the current position to the first character
                after the end delimiter */
@@ -533,7 +621,7 @@ char *ePerl_Bristled2Plain(char *cpBuf)
                 if (cps < cpEND)
                     cps++;
                 /* but preserve the newline in the script */
-                cpOut = ePerl_fprintf(cpOut, "\n");
+                cpOut = ePerl_fnprintf(cpOut, &cpOutLen, "\n");
             }
         }
     }
