@@ -38,35 +38,69 @@
 
 /*  first include the standard Perl 
     includes designed for embedding   */
+#define PERL_NO_GET_CONTEXT     /* for efficiency reasons, see perlguts(3) */
 #include <EXTERN.h>
 #include <perl.h>                 
-#include "ppport.h"
 
-
-/*  try to adjust for PerlIO handling  */
-#ifdef USE_PERLIO
-#undef  fwrite
-#define fwrite(buf,size,count,f) PerlIO_write(f,buf,size*count)
+#ifndef dTHR
+#  ifdef WIN32
+#       define dTHR extern int Perl___notused
+#  else
+#       define dTHR extern int errno
+#  endif
 #endif
 
+#ifndef aTHX
+#  define aTHX
+#  define aTHX_
+#  define pTHX void
+#  define pTHX_
+#endif
 
 /*  define the I/O type string for verbosity */
 #ifdef USE_PERLIO
-#ifdef USE_SFIO
-#define PERL_IO_LAYER_ID "PerlIO/SfIO"
+#  ifdef USE_SFIO
+#    define PERL_IO_LAYER_ID "PerlIO/SfIO"
+#  else
+#    define PERL_IO_LAYER_ID "PerlIO/StdIO"
+#  endif
 #else
-#define PERL_IO_LAYER_ID "PerlIO/StdIO"
+#  define PERL_IO_LAYER_ID "Raw/StdIO"
 #endif
-#else
-#define PERL_IO_LAYER_ID "Raw/StdIO"
+
+#if (PERL_VERSION < 4) || ((PERL_VERSION == 4) && (PERL_SUBVERSION <= 5))
+#  define PL_curstash curstash
 #endif
 
-/*   The PL_defoutgv is not in this version of ppport.h */
 #ifndef WITH_THR
-#define PL_defoutgv defoutgv
+#  define PL_defoutgv defoutgv
 #endif
 
-#include "ppport.h"
+/*  
+  Initialization of locales when building a new Perl interpreter.
+     Perl 5.003 calls perl_init_i18nl14n
+     Perl 5.004 and 5.005 call perl_init_i18nl10n
+     In Perl 5.6.0 this routine is already called by perl_construct
+*/
+#ifndef perl_init_i18nl10n
+#  define perl_init_i18nl10n perl_init_i18nl14n
+#else
+#  if (PERL_REVISION > 5) || ((PERL_REVISION == 5) && (PERL_VERSION >= 6))
+#    undef perl_init_i18nl10n
+#    define perl_init_i18nl10n(a)
+#  endif
+#endif
+
+/* eperl_perl5.c */
+/*  These prototypes can no longer be included in eperl_proto.h because
+    pTHX argument has been introduced in Perl 5.6.0  */
+extern void Perl5_XSInit(pTHX);
+extern void Perl5_ForceUnbufferedStdout(pTHX);
+extern char **Perl5_SetEnvVar(char **env, char *str);
+extern void Perl5_SetScalar(pTHX_ char *pname, char *vname, char *vvalue);
+extern char *Perl5_RememberedScalars[1024];
+extern void Perl5_RememberScalar(char *str);
+extern void Perl5_SetRememberedScalars(pTHX);
 
 #endif /* EPERL_PERL5_H */
 /*EOF*/
