@@ -15,7 +15,7 @@
 **
 **  ======================================================================
 **
-**  Copyright (c) 1996,1997,1998 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This program is free software; it may be redistributed and/or modified
 **  only under the terms of either the Artistic License or the GNU General
@@ -47,7 +47,8 @@ void ePerl_PP_SetError(char *str, ...)
     va_list ap;
 
     va_start(ap, str);
-    vsprintf(ePerl_PP_ErrorString, str, ap);
+    vsnprintf(ePerl_PP_ErrorString, sizeof(ePerl_PP_ErrorString), str, ap);
+    ePerl_PP_ErrorString[sizeof(ePerl_PP_ErrorString)-1] = NUL;
     va_end(ap);
     return;
 }
@@ -113,7 +114,8 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
             if (fp == NULL) {
                 /* we have to try in all include directories! */
                 for (i = 0; cppINC[i] != NULL; i++) {
-                    sprintf(caName, "%s/%s", cppINC[i], cpInput);
+                    snprintf(caName, sizeof(caName), "%s/%s", cppINC[i], cpInput);
+                    caName[sizeof(caName)-1] = NUL;
                     if ((fp = fopen(caName, "r")) != NULL)
                         break;
                 }
@@ -191,6 +193,7 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
             }
             cpOutBuf = cp2;
             strncpy(cpOutBuf+nOut, cps, cp-cps);
+            *(cpOutBuf+nOutBuf-1) = NUL;
             nOut += i;
 
             /* 
@@ -253,7 +256,7 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
                 }
 
                 /* copy the filename and skip to end of line */
-                for (i = 0; cps < cpEND && 
+                for (i = 0; i < sizeof(caName) && cps < cpEND && 
                             (*cps != ' ' && *cps != '\t' && 
                              *cps != '>' && *cps != '"'  &&
                              *cps != '\n'                  );  )
@@ -269,7 +272,7 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
                     return NULL;
 
                 /* make it secure by removing all begin/end delimiters!! */
-                if ((cp2 = (char *)malloc(strlen(cp))) == NULL)
+                if ((cp2 = (char *)malloc(strlen(cp)*9/8)) == NULL)
                     return NULL;
                 l1 = strlen(ePerl_begin_delimiter);
                 l2 = strlen(ePerl_end_delimiter);
@@ -278,6 +281,11 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
                         cp3 += l1;
                     else if (strncasecmp(cp3, ePerl_end_delimiter, l2) == 0)
                         cp3 += l2;
+                    else if (strncmp(cp3, "#include", 8) == 0) {
+                        /*  Replace all occurences of #include by #sinclude */
+                        strcpy(cp4, "#sinclude");
+                        cp4 += 9;
+                    }
                     else
                         *cp4++ = *cp3++;
                 }
@@ -301,13 +309,14 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
                 }
 
                 /* copy the argument and create replacement string */
-                for (i = 0; cps < cpEND && *cps != '\n'; )
+                for (i = 0; i < sizeof(caArg) && cps < cpEND && *cps != '\n'; )
                     caArg[i++] = *cps++;
                 caArg[i++] = NUL;
                 if (*cps == '\n')
                     cps++;
-                sprintf(caStr, "%s if (%s) { _%s//\n", 
+                snprintf(caStr, sizeof(caStr), "%s if (%s) { _%s//\n", 
                         ePerl_begin_delimiter, caArg, ePerl_end_delimiter);
+                caStr[sizeof(caStr)-1] = NUL;
                 cp = caStr;
             }
             else if (strncmp(cp, "#elsif", 6) == 0) {
@@ -326,13 +335,14 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
                 }
 
                 /* copy the argument and create replacement string */
-                for (i = 0; cps < cpEND && *cps != '\n'; )
+                for (i = 0; i < sizeof(caArg) && cps < cpEND && *cps != '\n'; )
                     caArg[i++] = *cps++;
                 caArg[i++] = NUL;
                 if (*cps == '\n')
                     cps++;
-                sprintf(caStr, "%s } elsif (%s) { _%s//\n", 
+                snprintf(caStr, sizeof(caStr), "%s } elsif (%s) { _%s//\n", 
                         ePerl_begin_delimiter, caArg, ePerl_end_delimiter);
+                caStr[sizeof(caStr)-1] = NUL;
                 cp = caStr;
             }
             else if (strncmp(cp, "#else", 5) == 0) {
@@ -348,8 +358,9 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
                     cps++;
 
                 /* create replacement string */
-                sprintf(caStr, "%s } else { _%s//\n", 
+                snprintf(caStr, sizeof(caStr), "%s } else { _%s//\n", 
                         ePerl_begin_delimiter, ePerl_end_delimiter);
+                caStr[sizeof(caStr)-1] = NUL;
                 cp = caStr;
             }
             else if (strncmp(cp, "#endif", 6) == 0) {
@@ -365,8 +376,9 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
                     cps++;
 
                 /* create replacement string */
-                sprintf(caStr, "%s } _%s//\n", 
+                snprintf(caStr, sizeof(caStr), "%s } _%s//\n", 
                         ePerl_begin_delimiter, ePerl_end_delimiter);
+                caStr[sizeof(caStr)-1] = NUL;
                 cp = caStr;
             }
             else if (strncmp(cp, "#c", 2) == 0) {
@@ -396,7 +408,8 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
                 return NULL;
             }
             cpOutBuf = cp2;
-            strcpy(cpOutBuf+nOut, cp);
+            strncpy(cpOutBuf+nOut, cp, nOutBuf - nOut);
+            *(cpOutBuf+nOutBuf-1) = NUL;
             nOut += i;
 
             continue;
@@ -413,7 +426,8 @@ char *ePerl_PP_Process(char *cpInput, char **cppINC, int mode)
                 return NULL;
             }
             cpOutBuf = cp2;
-            strcpy(cpOutBuf+nOut, cps);
+            strncpy(cpOutBuf+nOut, cps, nOutBuf - nOut);
+            *(cpOutBuf+nOutBuf-1) = NUL;
 
             break;
         }
