@@ -58,7 +58,7 @@
 #define SETUID_NEEDS_VALID_OWNER_UID     TRUE
 #define SETUID_NEEDS_VALID_OWNER_GID     TRUE
 #define SETUID_NEEDS_BELOW_OWNER_HOME    TRUE
-#define LIST_OF_ALLOWED_CALLER_UID       { "nobody", "root", NULL }
+#define LIST_OF_ALLOWED_CALLER_UID       { "nobody", "root", "www-data", NULL }
 
 /* 
  * Action when a SetUID security check failed.
