@@ -15,7 +15,7 @@
 **
 **  ======================================================================
 **
-**  Copyright (c) 1996,1997,1998 Ralf S. Engelschall <rse@engelschall.com>
+**  Copyright (c) 1996,1997,1998,1999 Ralf S. Engelschall <rse@engelschall.com>
 **
 **  This program is free software; it may be redistributed and/or modified
 **  only under the terms of either the Artistic License or the GNU General
@@ -37,6 +37,7 @@
 #include "eperl_global.h"
 #include "eperl_proto.h"
 
+char * getenv ();
 
 /*
 **
@@ -59,18 +60,18 @@ char **mysetenv(char **env, char *var, char *str, ...)
 
     /*  create the key=val string  */
     va_start(ap, str);
-    vsprintf(ca, str, ap);
-    va_end(ap);
-    sprintf(ca2, "%s=%s", var, ca);
+    vsnprintf(ca, sizeof(ca), str, ap);
+    ca[sizeof(ca)-1] = NUL;
+    snprintf(ca2, sizeof(ca2), "%s=%s", var, ca);
+    ca2[sizeof(ca2)-1] = NUL;
     cp = strdup(ca2);
 
     /*  now duplicate the old structure  */
     for (i = 0; env[i] != NULL; i++)
         ;
-    if ((envN = (char **)malloc(sizeof(char *) * (i+2))) == NULL)
-        return environ;
+    envN = (char **)malloc(sizeof(char *) * (i+2));
     for (i = 0; env[i] != NULL; i++) {
-        if (!replaced && (strncmp(env[i], var, strlen(var)) == 0)) {
+        if (strncmp(env[i], var, strlen(var)) == 0) {
             envN[i] = cp;
             replaced = TRUE;
         }
@@ -79,8 +80,9 @@ char **mysetenv(char **env, char *var, char *str, ...)
     }
 
     /*  add the new entry if not replaced */
-    if (!replaced)
+    if (!replaced) {
         envN[i++] = cp;
+    }
     envN[i] = NULL;
 
     /*  set the libc/exec variable which Perl uses */
@@ -89,6 +91,7 @@ char **mysetenv(char **env, char *var, char *str, ...)
     stillcalled = TRUE;
     environ = envN;
 
+    va_end(ap);
     return envN;
 }
 
@@ -210,9 +213,12 @@ char *mytmpfile(char *id)
     char *cp, *tmpdir;
     int i;
 
-    if ((tmpdir = getenv("TMPDIR")) == NULL)
-        tmpdir = "/tmp";
-    sprintf(ca, "%s/%s.%d.tmp%d", tmpdir, id, (int)getpid(), mytmpfilecnt++);
+    tmpdir = getenv ("TMPDIR");
+    if (tmpdir == (char *) NULL)
+        tmpdir="/tmp";
+
+    snprintf(ca, sizeof(ca), "%s/%s.%d.tmp%d", tmpdir, id, (int)getpid(), mytmpfilecnt++);
+    ca[sizeof(ca)-1] = NUL;
     cp = strdup(ca);
     for (i = 0; mytmpfiles[i] != NULL; i++)
         ;
@@ -268,7 +274,7 @@ char *ePerl_ReadSourceFile(char *filename, char **cpBufC, int *nBufC)
     if (stringEQ(filename, "-")) {
         /* file is given on stdin */
         ptr_tmpfile = mytmpfile("ePerl.source");
-        strcpy(tmpfile, ptr_tmpfile);
+        sprintf(tmpfile, "%s", ptr_tmpfile);
         if ((fp = fopen(tmpfile, "w")) == NULL) {
             ePerl_SetError("Cannot open temporary source file %s for writing", tmpfile);
             CU(NULL);
@@ -389,7 +395,8 @@ char *filename(char *path)
             ;
         if (cp == path+1)
             cp--;
-        strcpy(file, cp);
+        strncpy(file, cp, sizeof(file));
+        file[sizeof(file)-1] = '\0';
         return file;
     }
 }
@@ -402,7 +409,8 @@ char *dirname(char *path)
     if (path[strlen(path)-1] == '/')
         return path;
     else {
-        strcpy(dir, path);
+        strncpy(dir, path, sizeof(dir));
+        dir[sizeof(dir)-1] = '\0';
         for (cp = dir+strlen(dir); cp > dir && *(cp-1) != '/'; cp--)
             ;
         *cp = NUL;
@@ -428,9 +436,12 @@ char *abspath(char *path)
         /* restore cwd */
         chdir(cwd);
         /* add file part again */
-        if (apath[strlen(apath)-1] != '/')
-            strcpy(apath+strlen(apath), "/");
-        strcpy(apath+strlen(apath), path);
+        if (apath[strlen(apath)-1] != '/') {
+            strncpy(apath+strlen(apath), "/", sizeof(apath)-strlen(apath));
+            apath[sizeof(apath)-1] = '\0';
+        }
+        strncpy(apath+strlen(apath), path, sizeof(apath)-strlen(apath));
+        apath[sizeof(apath)-1] = '\0';
         return apath;
     }
 }
