@@ -1,7 +1,7 @@
 ##
 ##  slice_util.pl -- Utility functions
-##  Copyright (c) 1997-2001 Ralf S. Engelschall. 
-##  Copyright (c) 1999-2001 Denis Barbier.
+##  Copyright (c) 1997-2002 Ralf S. Engelschall. 
+##  Copyright (c) 1999-2002 Denis Barbier.
 ##
 
 package main;
@@ -29,6 +29,15 @@ sub error {
     exit(1);
 }
 
+sub fileerror {
+    my $file  = shift;
+    my ($str) = @_;
+
+    printerror($str);
+    unlink $file;
+    exit(1);
+}
+
 sub printwarning {
     my ($str) = @_;
 
